"""
Basic tests for randombytes_* functions
"""

import libnacl
import unittest


class TestRandomBytes(unittest.TestCase):
    def test_randombytes_random(self):
        self.assertIsInstance(libnacl.randombytes_random(), int)

    def test_randombytes_uniform(self):
        self.assertIsInstance(libnacl.randombytes_uniform(200), int)

        freq = {libnacl.randombytes_uniform(256): 1 for _ in range(65536)}

        self.assertEqual(256, len(freq))
        self.assertTrue(all(freq.values()))

    def test_randombytes(self):
        'copied from libsodium default/randombytes.c'

        data = libnacl.randombytes(65536)

        freq = {x: 1 for x in data}

        self.assertEqual(256, len(freq))
        self.assertTrue(all(freq.values()))



# -*- coding: utf-8 -*-
# Import libnacl libs
import libnacl.dual
import libnacl.secret
import libnacl.sign
import libnacl.utils

# Import pythonlibs
import os
import stat
import unittest
import tempfile
import sys

class TestSave(unittest.TestCase):
    '''
    '''
    def test_save_load(self):
        msg = b'then leap out of the rabbit, taking the French by surprise'
        bob = libnacl.dual.DualSecret()
        alice = libnacl.dual.DualSecret()
        fh_, bob_path = tempfile.mkstemp()
        os.close(fh_)
        fh_, alice_path = tempfile.mkstemp()
        os.close(fh_)
        bob.save(bob_path)
        alice.save(alice_path)
        bob_box = libnacl.public.Box(bob, alice.pk)
        alice_box = libnacl.public.Box(alice, bob.pk)
        bob_enc = bob_box.encrypt(msg)
        alice_enc = alice_box.encrypt(msg)
        bob_load = libnacl.utils.load_key(bob_path)
        alice_load = libnacl.utils.load_key(alice_path)
        bob_load_box = libnacl.public.Box(bob_load, alice_load.pk)
        alice_load_box = libnacl.public.Box(alice_load, bob_load.pk)
        self.assertEqual(bob.sk, bob_load.sk)
        self.assertEqual(bob.pk, bob_load.pk)
        self.assertEqual(bob.vk, bob_load.vk)
        self.assertEqual(bob.seed, bob_load.seed)
        self.assertEqual(alice.sk, alice_load.sk)
        self.assertEqual(alice.pk, alice_load.pk)
        self.assertEqual(alice.vk, alice_load.vk)
        self.assertEqual(alice.seed, alice_load.seed)
        bob_dec = alice_load_box.decrypt(bob_enc)
        alice_dec = bob_load_box.decrypt(alice_enc)
        self.assertEqual(bob_dec, msg)
        self.assertEqual(alice_dec, msg)

        bob2 = libnacl.utils.load_key(bob_path)
        self.assertEqual(bob.sk, bob2.sk)
        self.assertEqual(bob.pk, bob2.pk)
        self.assertEqual(bob.vk, bob2.vk)

        os.remove(bob_path)
        os.remove(alice_path)

    def test_save_load_secret(self):
        msg = b'then leap out of the rabbit, taking the French by surprise'
        box = libnacl.secret.SecretBox()
        fh_, box_path = tempfile.mkstemp()
        os.close(fh_)
        box.save(box_path)
        lbox = libnacl.utils.load_key(box_path)
        ctxt = box.encrypt(msg)
        out_msg = lbox.decrypt(ctxt)
        self.assertEqual(msg, out_msg)

    def test_save_load_sign(self):
        msg = b'then leap out of the rabbit, taking the French by surprise'
        signer = libnacl.sign.Signer()
        fh_, sign_path = tempfile.mkstemp()
        os.close(fh_)
        signer.save(sign_path)
        signer_load = libnacl.utils.load_key(sign_path)
        signed1 = signer.sign(msg)
        signed2 = signer_load.sign(msg)
        self.assertEqual(signed1, signed2)
        os.remove(sign_path)

    def test_save_perms(self):
        bob = libnacl.dual.DualSecret()
        fh_, bob_path = tempfile.mkstemp()
        os.close(fh_)
        bob.save(bob_path)
        stats = os.stat(bob_path)
        expected_perms = 0o100600 if sys.platform != 'win32' else 0o100666
        self.assertEqual(stats[stat.ST_MODE], expected_perms)
        os.remove(bob_path)



# Import libnacl libs
import libnacl.sign

# Import pythonlibs
import unittest


class TestSigning(unittest.TestCase):
    '''
    '''
    def test_sign(self):
        msg = (b'Well, that\'s no ordinary rabbit.  That\'s the most foul, '
               b'cruel, and bad-tempered rodent you ever set eyes on.')
        signer = libnacl.sign.Signer()
        signed = signer.sign(msg)
        signature = signer.signature(msg)
        self.assertNotEqual(msg, signed)
        veri = libnacl.sign.Verifier(signer.hex_vk())
        verified = veri.verify(signed)
        verified2 = veri.verify(signature + msg)
        self.assertEqual(verified, msg)
        self.assertEqual(verified2, msg)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Import python libs
import os
import sys
import unittest

NACL_ROOT = os.path.abspath(os.path.dirname(os.path.abspath(os.path.dirname(__file__))))
UNIT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), 'unit'))

sys.path.insert(0, NACL_ROOT)


def run_suite(path=UNIT_ROOT):
    '''
    Execute the unttest suite
    '''
    loader = unittest.TestLoader()
    tests = loader.discover(path)
    unittest.TextTestRunner(verbosity=2).run(tests)



if __name__ == '__main__':
    run_suite()



# Import nacl libs
import libnacl

# Import python libs
import unittest


class TestGenericHash(unittest.TestCase):
    '''
    Test sign functions
    '''
    def test_keyless_generichash(self):
        msg1 = b'Are you suggesting coconuts migrate?'
        msg2 = b'Not at all, they could be carried.'
        chash1 = libnacl.crypto_generichash(msg1)
        chash2 = libnacl.crypto_generichash(msg2)
        self.assertNotEqual(msg1, chash1)
        self.assertNotEqual(msg2, chash2)
        self.assertNotEqual(chash2, chash1)

    def test_key_generichash(self):
        msg1 = b'Are you suggesting coconuts migrate?'
        msg2 = b'Not at all, they could be carried.'
        key1 = libnacl.utils.rand_nonce()
        key2 = libnacl.utils.rand_nonce()
        khash1_1 = libnacl.crypto_generichash(msg1, key1)
        khash1_1_2 = libnacl.crypto_generichash(msg1, key1)
        khash1_2 = libnacl.crypto_generichash(msg1, key2)
        khash2_1 = libnacl.crypto_generichash(msg2, key1)
        khash2_2 = libnacl.crypto_generichash(msg2, key2)
        self.assertNotEqual(msg1, khash1_1)
        self.assertNotEqual(msg1, khash1_2)
        self.assertNotEqual(msg2, khash2_1)
        self.assertNotEqual(msg2, khash2_2)
        self.assertNotEqual(khash1_1, khash1_2)
        self.assertNotEqual(khash2_1, khash2_2)
        self.assertNotEqual(khash1_1, khash2_1)
        self.assertNotEqual(khash1_2, khash2_2)
        self.assertEqual(khash1_1, khash1_1_2)



# Import nacl libs
import libnacl
import libnacl.utils

# Import python libs
import unittest


class TestAuthVerify(unittest.TestCase):
    '''
    Test onetimeauth functions
    '''
    def test_auth_verify(self):
        msg = b'Anybody can invent a cryptosystem he cannot break himself. Except Bruce Schneier.'
        key1 = libnacl.utils.salsa_key()
        key2 = libnacl.utils.salsa_key()

        sig1 = libnacl.crypto_auth(msg, key1)
        sig2 = libnacl.crypto_auth(msg, key2)

        self.assertTrue(libnacl.crypto_auth_verify(sig1, msg, key1))
        self.assertTrue(libnacl.crypto_auth_verify(sig2, msg, key2))
        with self.assertRaises(ValueError) as context:
            libnacl.crypto_auth_verify(sig1, msg, key2)
        self.assertTrue('Failed to auth msg' in context.exception.args)

        with self.assertRaises(ValueError) as context:
            libnacl.crypto_auth_verify(sig2, msg, key1)
        self.assertTrue('Failed to auth msg' in context.exception.args)

    def test_onetimeauth_verify(self):
        self.assertEqual("poly1305", libnacl.crypto_onetimeauth_primitive())

        msg = b'Anybody can invent a cryptosystem he cannot break himself. Except Bruce Schneier.'
        key1 = libnacl.randombytes(libnacl.crypto_onetimeauth_KEYBYTES)
        key2 = libnacl.randombytes(libnacl.crypto_onetimeauth_KEYBYTES)

        sig1 = libnacl.crypto_onetimeauth(msg, key1)
        sig2 = libnacl.crypto_onetimeauth(msg, key2)

        with self.assertRaises(ValueError):
            libnacl.crypto_onetimeauth(msg, b'too_short')

        with self.assertRaises(ValueError):
            libnacl.crypto_onetimeauth_verify(sig1, msg, b'too_short')

        with self.assertRaises(ValueError):
            libnacl.crypto_onetimeauth_verify(b'too_short', msg, key1)

        self.assertTrue(libnacl.crypto_onetimeauth_verify(sig1, msg, key1))
        self.assertTrue(libnacl.crypto_onetimeauth_verify(sig2, msg, key2))
        with self.assertRaises(ValueError) as context:
            libnacl.crypto_onetimeauth_verify(sig1, msg, key2)
        self.assertTrue('Failed to auth message' in context.exception.args)

        with self.assertRaises(ValueError) as context:
            libnacl.crypto_onetimeauth_verify(sig2, msg, key1)
        self.assertTrue('Failed to auth message' in context.exception.args)




'''
Mimic very closely the python hashlib classes for blake2b

NOTE:
    This class does not yet implement streaming the msg into the
    hash function via the update method
'''

# Import python libs
import binascii

# Import libnacl libs
import libnacl


class Blake2b(object):
    '''
    Manage a Blake2b hash
    '''
    def __init__(self, msg, key=None):
        self.msg = msg
        self.key = key
        self.raw_digest = libnacl.crypto_generichash(msg, key)
        self.digest_size = len(self.raw_digest)

    def digest(self):
        '''
        Return the digest of the string
        '''
        return self.raw_digest

    def hexdigest(self):
        '''
        Return the hex digest of the string
        '''
        return binascii.hexlify(self.raw_digest)


def blake2b(msg, key=None):
    '''
    Create and return a Blake2b object to mimic the behavior of the python
    hashlib functions
    '''
    return Blake2b(msg, key)



"""
Basic tests for verify functions
"""

import libnacl
import unittest


# These are copied from libsodium test suite
class TestVerify(unittest.TestCase):
    def test_verify16(self):
        v16 = libnacl.randombytes_buf(16)
        v16x = v16[:]
        self.assertTrue(libnacl.crypto_verify_16(v16, v16x))
        self.assertTrue(libnacl.bytes_eq(v16, v16x))
        v16x = bytearray(v16x)
        v16x[libnacl.randombytes_random() & 15] += 1
        v16x = bytes(v16x)
        self.assertFalse(libnacl.crypto_verify_16(v16, v16x))
        self.assertFalse(libnacl.bytes_eq(v16, v16x))

        self.assertEqual(libnacl.crypto_verify_16_BYTES, 16)

    def test_verify32(self):
        v32 = libnacl.randombytes_buf(32)
        v32x = v32[:]
        self.assertTrue(libnacl.crypto_verify_32(v32, v32x))
        self.assertTrue(libnacl.bytes_eq(v32, v32x))
        v32x = bytearray(v32x)
        v32x[libnacl.randombytes_random() & 31] += 1
        v32x = bytes(v32x)
        self.assertFalse(libnacl.crypto_verify_32(v32, v32x))
        self.assertFalse(libnacl.bytes_eq(v32, v32x))

        self.assertEqual(libnacl.crypto_verify_32_BYTES, 32)

    def test_verify64(self):
        v64 = libnacl.randombytes_buf(64)
        v64x = v64[:]
        self.assertTrue(libnacl.crypto_verify_64(v64, v64x))
        self.assertTrue(libnacl.bytes_eq(v64, v64x))
        v64x = bytearray(v64x)
        v64x[libnacl.randombytes_random() & 63] += 1
        v64x = bytes(v64x)
        self.assertFalse(libnacl.crypto_verify_64(v64, v64x))
        self.assertFalse(libnacl.bytes_eq(v64, v64x))

        self.assertEqual(libnacl.crypto_verify_64_BYTES, 64)


class TestVerifyBytesEq(unittest.TestCase):
    def test_equal(self):
        a = libnacl.randombytes_buf(122)
        b = a[:]
        self.assertTrue(libnacl.bytes_eq(a, b))

    def test_different(self):
        a = libnacl.randombytes_buf(122)
        b = bytearray(a)
        b[87] += 1
        b = bytes(b)
        self.assertFalse(libnacl.bytes_eq(a, b))

    def test_invalid_type(self):
        a = libnacl.randombytes_buf(122)
        b = bytearray(a)
        with self.assertRaises(TypeError):
            libnacl.bytes_eq(a, b)

    def test_different_length(self):
        a = libnacl.randombytes_buf(122)
        b = a[:-1]
        self.assertFalse(libnacl.bytes_eq(a, b))



# -*- coding: utf-8 -*-
'''
Build in routines and classes to simplify encoding routines
'''
# Import python libs
import base64
import binascii


def hex_encode(data):
    '''
    Hex encode data
    '''
    return binascii.hexlify(data)


def hex_decode(data):
    '''
    Hex decode data
    '''
    return binascii.unhexlify(data)


def base16_encode(data):
    '''
    Base32 encode data
    '''
    return base64.b16encode(data)


def base16_decode(data):
    '''
    Base16 decode data
    '''
    return base64.b16decode(data)


def base32_encode(data):
    '''
    Base16 encode data
    '''
    return base64.b32encode(data)


def base32_decode(data):
    '''
    Base32 decode data
    '''
    return base64.b32decode(data)


def base64_encode(data):
    '''
    Base16 encode data
    '''
    return base64.b64encode(data)


def base64_decode(data):
    '''
    Base32 decode data
    '''
    return base64.b64decode(data)
